﻿package com.ludoko.pit
{
	import com.adamatomic.flixel.FlxEmitter;
	import com.adamatomic.flixel.FlxG;
	import com.adamatomic.flixel.FlxLayer;
	import com.adamatomic.flixel.FlxSprite;
	
	public class Bat extends FlxSprite
	{
		public static const SLEEP:uint 	= 0;
		public static const ACTIVE:uint = 1;
		public static const DEAD:uint 	= 2;
		
		[Embed(source = "../../../data/imgs/bat.png")] private var ImgBat:Class;
		[Embed(source = "../../../data/imgs/bat_gibs.png")] private var ImgBatGibs:Class;
		
		[Embed(source="../../../data/audio/bat_1.mp3")] private var SndBat1:Class;
		[Embed(source="../../../data/audio/bat_2.mp3")] private var SndBat2:Class;
		[Embed(source="../../../data/audio/bat_3.mp3")] private var SndBat3:Class;
		[Embed(source="../../../data/audio/bat_4.mp3")] private var SndBat4:Class;
		[Embed(source="../../../data/audio/bat_5.mp3")] private var SndBat5:Class;
		[Embed(source="../../../data/audio/bat_6.mp3")] private var SndBat6:Class;
		
		
		public var state:uint;
		public var sleepDuration:Number;
		
		private var _detectionRadius:int;
		private var _timer:Number;
		private var _player:Player;
		
		private var _hits:int;
		
		private var _gibs:FlxEmitter;
		
		public function Bat(X:int, Y:int, ThePlayer:Player, GibLayer:FlxLayer) 
		{
			super(ImgBat, X, Y, true, false);
			
			maxAngular = 120;
			angularDrag = 400;
			maxThrust = 100;
			drag.x = 40;
			drag.y = 40;
			
			state = SLEEP;
			sleepDuration = 3 + Math.random() * 2;
			
			_detectionRadius = 1000;
			_timer = 0;
			_player = ThePlayer;
			
			_hits = 0;
			
			_gibs = GibLayer.add(new FlxEmitter(0, 0, 0, 0, null, -1.5, -150, 150, -200, 0, 0, 0, 400, 0, ImgBatGibs, 10, true)) as FlxEmitter;
			
			addAnimation("idle", [0]);
			addAnimation("flying", [1, 2], 15, true);
			addAnimation("dead", [3]);
			play("idle");
		}
		
		override public function update():void
		{
			if (dead || state == DEAD)
			{
				if (!onScreen() && y > FlxG.height) 
					exists = false;
				else
					super.update();
					
				//trace(velocity.x + " " + velocity.y);
				return;
			}
			
			_timer += FlxG.elapsed;
			if (state == SLEEP)
			{
				if (_timer > sleepDuration && onScreen())
				{
					state = ACTIVE;
					play("flying");
					_timer = 0;
				}
			}
			else if (state == ACTIVE)
			{
				var dx:Number = x - _player.x;
				var dy:Number = y - _player.y;
				var da:Number = FlxG.getAngle(dx, dy);
				if (da < 0)
					da += 360;
				var ac:Number = angle;
				if (ac < 0)
					ac += 360;
				if (da < angle)
					angularAcceleration = -angularDrag;
				else if(da > angle)
					angularAcceleration = angularDrag;
				else
					angularAcceleration = 0;
				
				thrust = 15;
				if (_timer > 6)
					_timer = 0;
				else if (_timer < 4)
					thrust = 25;
			}
			
			super.update();
		}
		
		override public function render(Angle:Number = -1000):void
		{
			super.render(0);
		}
		
		override public function kill():void
		{
			if (dead || flickering())
				return;
			
			PlayState(FlxG.state).incrementDashHits(x, y);
			
			if (_hits >= 5)
			{
				PlayState(FlxG.state).playerHeal(x, y);
				
				super.kill();
				exists = true;
			}
			
			//FlxG.play(SndExplode);
			
			flicker(0.35);
			play("dead");
			
			state = DEAD;
			
			_gibs.x = x + width / 2;
			_gibs.y = y + height / 2;
			_gibs.reset();
			
			// Let's start juggling
			thrust = 0;
			angularVelocity = 0;
			angularAcceleration = 0;
			angularDrag = 0;
			angle = 0;
			
			acceleration.x = 0;
			acceleration.y = 400;
			velocity.x = -25 + Math.random() * 50;
			velocity.y = -215;
			maxVelocity.x = maxVelocity.y = 10000;
			drag.x = 0;
			drag.y = 0;
			
			switch (_hits) 
			{
				case 0:
					PlayState(FlxG.state).scoreToAdd += 100;
					PlayState(FlxG.state).addPoints(x, y, 100);
					FlxG.play(SndBat1);
					break;
				case 1:
					PlayState(FlxG.state).scoreToAdd += 200;
					PlayState(FlxG.state).addPoints(x, y, 200);
					FlxG.play(SndBat2);
					break;
				case 2:
					PlayState(FlxG.state).scoreToAdd += 400;
					PlayState(FlxG.state).addPoints(x, y, 400);
					FlxG.play(SndBat3);
					break;
				case 3:
					PlayState(FlxG.state).scoreToAdd += 800;
					PlayState(FlxG.state).addPoints(x, y, 800);
					FlxG.play(SndBat4);
					break;
				case 4:
					PlayState(FlxG.state).scoreToAdd += 1600;
					PlayState(FlxG.state).addPoints(x, y, 1600);
					FlxG.play(SndBat5);
					break;
				case 5:
					PlayState(FlxG.state).scoreToAdd += 3200;
					PlayState(FlxG.state).addPoints(x, y, 3200);
					FlxG.play(SndBat6);
					break;
			}
			
			_hits++;
		}
		
		public function silentKill():void
		{
			if (dead) return;
			
			//FlxG.play(SndExplode);
			super.kill();
			exists = false;
		}
		
		public function reset(X:int, Y:int):void
		{
			exists = true;
			dead = false;
			visible = true;
			active = true;
			
			x = X;
			y = Y;
			
			thrust = 0;
			velocity.x = 0;
			velocity.y = 0;
			angle = 0;
			
			_timer = 0;
			state = SLEEP;
			sleepDuration = 3 + Math.random() * 2;
			
			_hits = 0;
			
			_gibs.kill();
			
			play("idle");
		}
	}
}