﻿package com.ludoko.pit
{
	import com.adamatomic.flixel.FlxArray;
	import com.adamatomic.flixel.FlxBlock;
	import com.adamatomic.flixel.FlxEmitter;
	import com.adamatomic.flixel.FlxG;
	import com.adamatomic.flixel.FlxLayer;
	import com.adamatomic.flixel.FlxState;
	import com.adamatomic.flixel.FlxSprite;
	import com.adamatomic.flixel.FlxText;
	import flash.geom.Point;
	import flash.text.TextFormatAlign;
	
	public class PlayState extends FlxState
	{
		[Embed(source="../../../data/imgs/bg.png")] private var ImgBG:Class;
		[Embed(source="../../../data/imgs/wall.png")] private var ImgWall:Class;
		[Embed(source="../../../data/imgs/rope.png")] private var ImgRope:Class;
		[Embed(source="../../../data/imgs/player/dash.png")] private var ImgDash:Class;
		[Embed(source="../../../data/imgs/heart.png")] private var ImgHeart:Class;
		
		[Embed(source="../../../data/audio/health.mp3")] private var SndHealth:Class;
		[Embed(source="../../../data/audio/rumble.mp3")] private var SndRumble:Class;
		
		
		private var _bgs:FlxArray;
		private var _bgLayer:FlxLayer;
		
		private var _walls:FlxArray;
		private var _wallLayer:FlxLayer;
		
		private var _enemies:FlxArray;
		private var _enemiesLayer:FlxLayer;
		
		private var _traps:FlxArray;
		private var _trapsLayer:FlxLayer;
		
		private var _rope:FlxSprite;
		private var _player:Player;
		private var _dash:FlxArray;
		private var _playerLayer:FlxLayer;
		
		private var _hud:FlxLayer;
		private var _hearts:FlxArray;
		
		private var _points:FlxArray;
		private var _combos:FlxArray;
		private var _health:Health;
		
		private var _level:int;
		private var _distanceToNextLevel:Number;
		
		private var _diveSpeed:Number;
		private var _depth:Number;
		
		public var scoreToAdd:int;
		private var _dashHits:int;
		
		private var _quakeMagnitude:Number;
		
		private var _doSpawnEnemies:Boolean;
		private var _enemiesWaveSize:int, _maxEnemiesWaveSize:int;
		private var _enemySpawnTimer:Number, _enemySpawnWait:Number, _minEnemySpawnWait:Number;
		
		private var _doSpawnTraps:Boolean;
		private var _trapLength:int, _maxTrapLength:int;
		private var _trapSpawnTimer:Number, _trapSpawnWait:Number, _minTrapSpawnWait:Number;
		
		private var _scoreText:FlxText;
		private var _depthText:FlxText;
		
		private var _fading:Boolean;
		
		public function PlayState() 
		{
			super();
			
			var i:int;
			
			_bgs = new FlxArray();
			_bgLayer = this.add(new FlxLayer) as FlxLayer;
			_bgs.add(_bgLayer.add(new FlxBlock(0, 0, FlxG.width, FlxG.height, ImgBG, int(Math.random() * 40))));
			_bgs.add(_bgLayer.add(new FlxBlock(0, FlxG.height, FlxG.width, FlxG.height, ImgBG, int(Math.random() * 40))));
			
			_walls = new FlxArray();
			_wallLayer = this.add(new FlxLayer) as FlxLayer;
			_walls.add(_wallLayer.add(new FlxBlock(0, 0, 16, FlxG.height, ImgWall)));
			_walls.add(_wallLayer.add(new FlxBlock(FlxG.width - 16, 0, 16, FlxG.height, ImgWall)));
			_walls.add(_wallLayer.add(new FlxBlock(0, FlxG.height, 16, FlxG.height, ImgWall)));
			_walls.add(_wallLayer.add(new FlxBlock(FlxG.width - 16, FlxG.height, 16, FlxG.height, ImgWall)));
			
			_enemies = new FlxArray();
			_traps = new FlxArray();
			
			_player = new Player(112, 152);
			_rope = new FlxSprite(ImgRope, _player.x + 7, _player.y - 236, false, false);
			
			_enemiesLayer = this.add(new FlxLayer()) as FlxLayer;
			_trapsLayer = this.add(new FlxLayer()) as FlxLayer;
			
			_playerLayer = this.add(new FlxLayer) as FlxLayer;
			_playerLayer.add(_rope);
			_dash = new FlxArray();
			_playerLayer.add(_player);
			for (i = 0; i < 8; i++)
			{
				var sprite:FlxSprite = new FlxSprite(ImgDash, 0, 0, true, true);
				sprite.alpha = 0.6 - i / 16;
				sprite.addAnimation("dash", [0]);
				sprite.play("dash");
				_dash.add(_playerLayer.add(sprite));
				_dash[i].kill();
			}
			
			_hud = this.add(new FlxLayer()) as FlxLayer;
			
			_points = new FlxArray();
			_combos = new FlxArray();
			_health = _hud.add(new Health(0, 0)) as Health;
			_health.kill();
			
			_hearts = new FlxArray();
			var heart:FlxSprite;
			for (i = 0; i < 5; i++)
			{
				heart = new FlxSprite(ImgHeart, 21 + i * 11, 4, true, false);
				heart.scrollFactor.x = heart.scrollFactor.y = 0;
				heart.addAnimation("on", [0]);
				heart.addAnimation("off", [1]);
				heart.play("on");
				_hearts.add(_hud.add(heart));
			}
			
			_level = 1;
			_distanceToNextLevel = 100;
			
			_diveSpeed = 50;
			_depth = 0;
			
			_dashHits = 0;
			
			scoreToAdd = 0;
			
			_doSpawnEnemies = false;
			_enemiesWaveSize = 2;
			_maxEnemiesWaveSize = 8;
			_enemySpawnWait = 8;
			_enemySpawnTimer = _enemySpawnWait / 2;
			_minEnemySpawnWait = 3;
			
			_doSpawnTraps = true;
			_trapLength = 4;
			_maxTrapLength = 14;
			_trapSpawnWait = 5;
			_trapSpawnTimer = _trapSpawnWait / 2;
			_minTrapSpawnWait = 1;
			
			_quakeMagnitude = 0.001;
			
			_scoreText = this.add(new FlxText(FlxG.width - 70, 2, 50, 20, "0", 0xffffff, null, 8, TextFormatAlign.RIGHT)) as FlxText;
			_depthText = this.add(new FlxText(FlxG.width - 70, FlxG.height - 20, 50, 20, "0 M", 0xffffff, null, 8, TextFormatAlign.RIGHT)) as FlxText;
			
			_fading = false;
			
			FlxG.score = 0;
			
			FlxG.setMusic(SndRumble, 0.7, true);
			FlxG.flash(0xffffffff);
		}
		
		override public function update():void
		{
			var i:int;
			var score:Number = FlxG.score;
			
			super.update();
			
			
			// Enemy spawn timer
			_enemySpawnTimer -= FlxG.elapsed;
			if (_enemySpawnTimer <= 0 && _doSpawnEnemies)
			{
				for (i = 0; i < _enemiesWaveSize; i++)
					spawnBat(Math.random() * FlxG.width, FlxG.height + Math.random() * FlxG.height / 2);
				_enemySpawnTimer = _enemySpawnWait;
			}
			
			
			// Trap spawn timer
			_trapSpawnTimer -= FlxG.elapsed;
			if (_trapSpawnTimer <= 0 && _doSpawnTraps)
			{
				var startX:int = int(Math.random() * FlxG.width / 16) * 16;
				var startY:int = FlxG.height + Math.random() * FlxG.height / 2;
				for (i = 0; i < _trapLength; i++)
					spawnTrap(startX + i * 16, startY);
				_trapSpawnTimer = _trapSpawnWait;
			}
			
			
			// Collision handling
			FlxG.overlapArray(_walls, _player, playerHitsWall);
			FlxG.overlapArray(_enemies, _player, enemyHitsPlayer);
			FlxG.overlapArray(_traps, _player, trapHitsPlayer);
			FlxG.overlapArrays(_walls, _enemies, enemyHitsWall);
			
			FlxG.collideArray(_walls, _player);
			
			
			// Scroll level
			var block:FlxBlock;
			for (i = 0; i < _bgs.length; i++)
			{
				block = _bgs[i];
				block.y -= _diveSpeed * FlxG.elapsed / 10;
				if (block.y <= -FlxG.height)
				{
					_bgs[i] = _bgLayer.add(new FlxBlock(0, block.y + FlxG.height * 2 - 1, FlxG.width, FlxG.height, ImgBG, int(Math.random() * 40)));
					_bgLayer.remove(block);
				}
			}
			
			for (i = 0; i < _walls.length; i++)
			{
				block = _walls[i];
				block.y -= _diveSpeed * FlxG.elapsed;
				if (block.y <= -FlxG.height)
				{
					_walls[i] = _wallLayer.add(new FlxBlock(block.x, block.y + FlxG.height * 2 - 1, 16, FlxG.height, ImgWall));
					_wallLayer.remove(block);
				}
			}
			
			var bat:Bat;
			for (i = 0; i < _enemies.length; i++)
			{
				bat = _enemies[i];
				if (bat.active && bat.exists && bat.state == Bat.SLEEP)
				{
					bat.y -= _diveSpeed * FlxG.elapsed;
					if (bat.y <= -FlxG.height)
						bat.silentKill();
				}
			}
			
			var trap:Trap;
			for (i = 0; i < _traps.length; i++)
			{
				trap = _traps[i];
				if (trap.active && trap.exists)
				{
					trap.y -= _diveSpeed * FlxG.elapsed;
					if (trap.y <= -FlxG.height)
						trap.kill();
				}
			}
			
			
			// Adjust rope to player
			if (!_player.dead)
			{
				_rope.x = _player.x + 7;
				_rope.y = _player.y - 316;
			}
			
			if (!_player.dead)
				_depth += _diveSpeed / 10 * FlxG.elapsed;
			if (_depth >= _distanceToNextLevel)
			{
				_level++;
				
				if (_level < 5)
				{
					_doSpawnEnemies = !_doSpawnEnemies;
					_doSpawnTraps = !_doSpawnTraps;
				}
				else
				{
					_doSpawnEnemies = true;
					_doSpawnTraps = true;
				}
				
				if (_doSpawnEnemies)
				{
					_enemiesWaveSize++; 
					if (_enemiesWaveSize > _maxEnemiesWaveSize)
						_enemiesWaveSize = _maxEnemiesWaveSize;
					_enemySpawnWait -= 0.5;
					if (_enemySpawnWait < _minEnemySpawnWait)
						_enemySpawnWait = _minEnemySpawnWait;
				}
				if (_doSpawnTraps)
				{
					_trapLength++;
					if (_trapLength > _maxTrapLength)
						_trapLength = _maxTrapLength;
					
					_trapSpawnWait -= 0.5;
					if (_trapSpawnWait < _minTrapSpawnWait)
						_trapSpawnWait = _minTrapSpawnWait;
				}
				
				_distanceToNextLevel += 100;
				
				_quakeMagnitude += 0.001;
				if (_quakeMagnitude > 0.007)
					_quakeMagnitude = 0.007;
			}
			
			// Update dash (if necessary)
			var sprite:FlxSprite;
			if (_player.dashing)
			{
				for (i = 0; i < _dash.length; i++)
				{
					sprite = _dash[i];
					if (!sprite.exists)
					{
						sprite.exists = true;
						sprite.dead = false;
						break;
					}
				}
				
				for (i = _dash.length - 1; i >= 0; i--)
				{
					sprite = _dash[i];
					if (i != 0)
					{
						sprite.x = _dash[i - 1].x;
						sprite.y = _dash[i - 1].y;
					}
					else
					{
						sprite.x = _player.x;
						sprite.y = _player.y;
					}
					sprite.facing = _player.facing;
				}
			}
			else
				for (i = _dash.length - 1; i >= 0; i--)
					if (_dash[i].exists)
					{
						_dash[i].kill();
						break;
					}
			
			if(score != FlxG.score)
			{
				if (_player.dead) FlxG.score = 0;
				_scoreText.setText(FlxG.score.toString());
			}
			
			if (!_player.dead)
				_depthText.setText(_depth.toFixed(0) + " M");
			
			if (Math.random() < 0.03)
				FlxG.quake(_quakeMagnitude, 0.2);
		}
		
		public function onDeath():void
		{
			FlxG.stopMusic();
			FlxG.scores[0] = FlxG.score;
			FlxG.scores[1] = _depth;
			FlxG.switchState(GameOverState);
		}
		
		private function spawnBat(X:int, Y:int):void
		{
			var bat:Bat;
			for (var i:int = 0; i < _enemies.length; i++)
			{
				bat = _enemies[i];
				if (bat.dead && bat.finished)
				{
					bat.reset(X, Y);
					return;
				}
			}
			
			bat = new Bat(X, Y, _player, _enemiesLayer);
			_enemies.add(_enemiesLayer.add(bat));
		}
		
		private function spawnTrap(X:int, Y:int):void
		{
			var trap:Trap;
			for (var i:int = 0; i < _traps.length; i++)
			{
				trap = _traps[i];
				if (trap.dead)
				{
					trap.reset(X, Y);
					return;
				}
			}
			
			_traps.add(_trapsLayer.add(new Trap(X, Y)));
		}
		
		public function addPoints(X:int, Y:int, Points:int):void
		{
			var points:ScorePoints;
			for (var i:int = 0; i < _points.length; i++)
			{
				points = _points[i];
				if (points.dead)
				{
					points.reset(X, Y, Points);
					return;
				}
			}
			
			_points.add(_hud.add(new ScorePoints(X, Y, Points)));
		}
		
		public function addCombo(X:int, Y:int, Combo:int):void
		{
			var combo:ScoreCombos;
			for (var i:int = 0; i < _combos.length; i++)
			{
				combo = _combos[i];
				if (combo.dead)
				{
					combo.reset(X, Y, Combo);
					return;
				}
			}
			
			_combos.add(_hud.add(new ScoreCombos(X, Y, Combo)));
		}
		
		private function playerHitsWall(Wall:FlxBlock, ThePlayer:Player):void
		{
			ThePlayer.reverse(Wall);
		}
		
		private function enemyHitsPlayer(ABat:Bat, ThePlayer:Player):void
		{
			// If player is attacking then kill the bat
			if (_player.dashing)
				ABat.kill();
			else if (ABat.state != Bat.DEAD && !ThePlayer.flickering())
				playerHurt();
		}
		
		private function trapHitsPlayer(ATrap:Trap, ThePlayer:Player):void
		{
			if (ThePlayer.flickering())
				return;
			
			playerHurt();
		}
		
		private function enemyHitsWall(AWall:FlxBlock, ABat:Bat):void 
		{
			// Only care about bat collisions when they're dead
			if (ABat.state != Bat.DEAD)
				return;
			
			// Reverse velocity to fake wall collision
			ABat.velocity.x = -ABat.velocity.x;
		}
		
		public function resetDashHits():void 
		{ 
			FlxG.score += scoreToAdd * _dashHits;
			scoreToAdd = 0;
			
			_dashHits = 0;
		}
		
		public function incrementDashHits(X:int, Y:int):void 
		{ 
			_dashHits++;
			if (_dashHits > 1)
				addCombo(X, Y, _dashHits);
		}
		
		private function playerHurt():void
		{
			_player.hurt(0);
			
			for (var i:int = _hearts.length - 1; i >= 0; i--)
			{
				var heart:FlxSprite = _hearts[i];
				if (heart.currentAnimation == "on")
				{
					heart.play("off");
					
					if (i == 0)
						_player.kill();
					else
						_player.flicker();
					
					break;
				}
			}
		}
		
		public function playerHeal(X:int, Y:int):void
		{
			for (var i:int = 0; i < _hearts.length; i++)
			{
				var heart:FlxSprite = _hearts[i];
				if (heart.currentAnimation == "off")
				{
					heart.play("on");
					_health.reset(X, Y);
					FlxG.play(SndHealth);
					break;
				}
			}
		}
	}
}