﻿package com.ludoko.pit
{
	import com.adamatomic.flixel.FlxCore;
	import com.adamatomic.flixel.FlxEmitter;
	import com.adamatomic.flixel.FlxG;
	import com.adamatomic.flixel.FlxSprite;
	
	public class Player extends FlxSprite
	{
		[Embed(source="../../../data/imgs/player/player.png")] private var ImgPlayer:Class;
		
		[Embed(source="../../../data/audio/dash.mp3")] private var SndDash:Class;
		[Embed(source = "../../../data/audio/wall_collide_1.mp3")] private var SndWallCollide:Class;
		[Embed(source="../../../data/audio/wall_collide_2.mp3")] private var SndWallCollideDash:Class;
		[Embed(source="../../../data/audio/hurt.mp3")] private var SndHurt:Class;
		[Embed(source = "../../../data/audio/splat.mp3")] private var SndSplat:Class;
		
		
		// Swinging variables
		public var swinging:Boolean, swingDirection:Boolean;
		private var _swingAccel:Number, _maxSwingAccel:Number, _swingImpulse:Number, _swingAngle:Number, _swingPeriod:Number, _swingTimer:Number;
		
		private var _touchingWall:Boolean;
		private var _touchingWallTimer:Number;
		
		// Control variables
		private var _kDoubleTapLeft:Boolean, _kDoubleTapRight:Boolean, _kLookForDoubleTap:Boolean, _kInitialTap:Boolean;
		private var _doubleTapTimer:Number = 0;
		
		public var dashing:Boolean;
		private var _dash:FlxEmitter;
		private var _dashTimer:Number, _dashDuration:Number, _dashCooldown:Number, _dashWait:Number;
		
		private var _restart:Number;
		private var _falling:Boolean;
		
		public function Player(X:int, Y:int) 
		{
			super(ImgPlayer, X, Y, true, true);
			
			offset.x = 0;
			width = 16;
			
			//basic player physics
			var runSpeed:uint = 80;
			drag.x = runSpeed * 4;
			drag.y = runSpeed * 8;
			maxVelocity.x = runSpeed;
			maxVelocity.y = runSpeed;
			
			swinging = false;
			swingDirection = LEFT;
			
			_swingAccel = 0;
			_maxSwingAccel = 100;
			_swingImpulse = 0;
			_swingAngle = 60;
			_swingPeriod = 4;
			_swingTimer = _swingPeriod / 2;
			
			_touchingWall = false;
			_touchingWallTimer = 0;
			
			_kDoubleTapLeft = _kDoubleTapRight = _kLookForDoubleTap = false;
			_kInitialTap = LEFT;
			_doubleTapTimer = 0;
			
			_dashTimer = 0;
			_dashDuration = 0.55;
			_dashCooldown = 0;
			_dashWait = 0.75;
			
			_restart = 0;
			_falling = false;
			
			addAnimation("idle", [0]);
			addAnimation("dash", [1], 5, false);
			play("idle");
		}
		
		override public function update():void
		{
			// If dead start restart timer
			if (dead)
			{
				if (_falling)
				{
					if (y < FlxG.height * 2)
						super.update();
					else
					{
						FlxG.quake(0.005,0.35);
						FlxG.play(SndSplat);
						_falling = false;
					}
				}
				
				_restart += FlxG.elapsed;
				if (_restart > 4)
					PlayState(FlxG.state).onDeath();
				return;
			}
			
			// Otherwise continue on
			
			_dashTimer -= FlxG.elapsed;
			if (dashing && _dashTimer <= 0)
			{
				PlayState(FlxG.state).resetDashHits();
				dashing = false;
				play("idle");
			}
			
			// Double-tap detection
			var reversedTap:Boolean = false;
			_doubleTapTimer += FlxG.elapsed;
			if (_kLookForDoubleTap)
			{
				if (FlxG.justPressed(FlxG.LEFT))
				{
					if (_kInitialTap == LEFT)
						_kDoubleTapLeft = true;
					else
					{
						_kLookForDoubleTap = false;
						_doubleTapTimer = 0;
						reversedTap = true;
					}
				}
				if (FlxG.justPressed(FlxG.RIGHT))
				{
					if (_kInitialTap == RIGHT)
						_kDoubleTapRight = true;
					else
					{
						_kLookForDoubleTap = false;
						_doubleTapTimer = 0;
						reversedTap = true;
					}
				}
				
				if (_doubleTapTimer > 0.2 || _kDoubleTapLeft || _kDoubleTapRight)
				{
					_kLookForDoubleTap = false;
					_doubleTapTimer = 0;
				}
			}
			
			if (!_kLookForDoubleTap)
			{
				if (_doubleTapTimer > 0.2 || reversedTap) // Double tap cooldown
				{
					if (FlxG.justPressed(FlxG.LEFT) && !FlxG.kRight)
					{
						_kInitialTap = LEFT;
						_kLookForDoubleTap = true;
						_doubleTapTimer = 0;
					}
					if (FlxG.justPressed(FlxG.RIGHT) && !FlxG.kLeft)
					{
						
						_kInitialTap = RIGHT;
						_kLookForDoubleTap = true;
						_doubleTapTimer = 0;
					}
				}
			}
			
			
			// Handle swinging
			if (_swingAccel == 0)
			{
				if (FlxG.kLeft)
				{
					_swingAccel = 10;
					_swingTimer = _swingPeriod / 2;
					swingDirection = LEFT;
				}
				else if (FlxG.kRight)
				{
					_swingAccel = 10;
					_swingTimer = _swingPeriod;
					swingDirection = RIGHT;
				}
			}
			
			// Handle double-tap swinging
			_dashCooldown -= FlxG.elapsed;
			
			_swingImpulse *= 0.96;
			if (_swingImpulse < 1) _swingImpulse = 0;
			if (_dashCooldown <= 0)
			{
				if (_kDoubleTapLeft)
				{
					_swingImpulse = 110;
					_swingTimer = _swingPeriod / 2;
					swingDirection = LEFT;
					dash(LEFT);
				}
				else if (_kDoubleTapRight)
				{
					_swingImpulse = 120;
					_swingTimer = _swingPeriod;
					swingDirection = RIGHT;
					dash(RIGHT);
				}
			}
			
			if (swingDirection == LEFT)
			{
				_swingTimer += FlxG.elapsed;
				if (_swingTimer >= _swingPeriod)
				{
					swingDirection = !swingDirection;
					_swingTimer = _swingPeriod;
					_swingAccel *= 0.7;
				}
			}
			else
			{
				_swingTimer -= FlxG.elapsed;
				if (_swingTimer <= 0)
				{
					swingDirection = !swingDirection;
					_swingTimer = 0;
					_swingAccel *= 0.7;
				}
			}
			
			if (_swingAccel < 5) _swingAccel = 0;
			
			acceleration.x = 0;
			var swingAngle:Number = (_swingAngle * Math.PI / 180) * Math.cos(2 * Math.PI * _swingTimer / _swingPeriod);
			var tan:Number = Math.tan(swingAngle);
			
			// This is hacky, but I can't be bothered with a better solution right now
			if (tan < 0) // Going left
			{
				if (FlxG.kLeft)
					_swingAccel += 0.15;
				else if (FlxG.kRight)
					_swingAccel -= 0.3;
			}
			else // Going right
			{
				if (FlxG.kRight)
					_swingAccel += 0.15;
				else if (FlxG.kLeft)
					_swingAccel -= 0.3;
			}
			
			if (_swingAccel < 0) _swingAccel = 0;
			else if (_swingAccel > _maxSwingAccel) _swingAccel = _maxSwingAccel;
			
			var swingVel:Number = (_swingAccel + _swingImpulse) * tan * FlxG.elapsed;
			//trace((_swingTimer / _swingPeriod) + " " + swingVel);
			x += swingVel;
			
			// Move left and right
			if(FlxG.kLeft)
			{
				facing = LEFT;
				acceleration.x -= drag.x;
			}
			else if(FlxG.kRight)
			{
				facing = RIGHT;
				acceleration.x += drag.x;
			}
			
			// Move up and down
			acceleration.y = 0;
			if(FlxG.kUp)
				acceleration.y -= drag.y;
			else if(FlxG.kDown)
				acceleration.y += drag.y;
			
			// Another hack to keep the player in bounds
			if (x < 16) x = 16;
			else if (x > FlxG.width - 32) x = FlxG.width - 32;
			
			if (y < 0) y = 0;
			else if (y > FlxG.height - 16) y = FlxG.height - 16;
			
			super.update();
			
			// Reset flags
			_kDoubleTapLeft = _kDoubleTapRight = false;
			
			if (_touchingWall)
			{
				_touchingWallTimer -= FlxG.elapsed;
				if (_touchingWallTimer <= 0)
					_touchingWall = false; 
			}
		}
		
		override public function render(Angle:Number = -1000):void
		{
			super.render();
		}
		
		override public function hitWall():Boolean
		{
			return super.hitWall();
		}
		
		override public function hurt(Damage:Number):void
		{
			FlxG.play(SndHurt);
			super.hurt(0);
		}
		
		override public function kill():void
		{
			if (dead) return;
			
			//FlxG.play(SndExplode);
			//FlxG.play(SndExplode2);
			
			super.kill();
			flicker(-1);
			exists = true;
			
			// Let's start falling!
			thrust = 0;
			angularVelocity = -720 + Math.random() * 1440;
			angularAcceleration = 0;
			angularDrag = 0;
			angle = 0;
			
			acceleration.x = 0;
			acceleration.y = 400;
			velocity.x = 0;
			velocity.y = 0;
			maxVelocity.x = maxVelocity.y = 10000;
			drag.x = 0;
			drag.y = 0;
			
			FlxG.flash(0xffff0000, 0.35);
			
			_restart = 0;
			_falling = true;
			
			//_gibs.x = x + width / 2;
			//_gibs.y = y + height / 2;
			//_gibs.reset();
		}
		
		private function dash(Direction:Boolean):void 
		{
			if (dashing) return;
			
			dashing = true;
			_dashTimer = _dashDuration;
			_dashCooldown = _dashWait;
			play("dash");
			FlxG.play(SndDash);
		}
		
		public function reverse(Core:FlxCore):void
		{
			if (_touchingWall) return;
			
			velocity.x = 0;
			_touchingWall = true;
			if (dashing)
				FlxG.play(SndWallCollideDash);
			//else
			//	FlxG.play(SndWallCollide, 0.6);
			_touchingWallTimer = 0.8;
		}
	}
	
}